import os
import json
import pydicom
import time
from pydicom.uid import generate_uid

FILL_NUMBER = 3

def create_anonymization_table(json_path):
    """ Create a anonymization table (json).

    Parameters
    ----------
    json_path : str
        The path of the JSON table file

    """
    with open(json_path, 'x') as f:
        data = {"anonymisation" : []}
        json.dump(data, f, indent=4)

def write(json_path, sub, patient_id):
    """ Write new patient in the anonymization table.

    Parameters
    ----------
    json_path : str
        The path of the JSON table file

    sub : str
        The key (unique id) of the new patient in bids format (ex: sub-001)

    patient_id : str
        The value (IPP) of the new patient (ex: firstname^lastname^IPP)
    """
    with open(json_path) as f:
        data = json.load(f)

    anonymisationDict = dict()
    anonymisationDict[sub] = patient_id
    data["anonymisation"].append(anonymisationDict)
        
    with open(json_path, 'w') as f:
        json.dump(data, f, indent=4)
   
def get_sub(json_path, patient_id):
    """ Create the anonymization table if it is not existing or get the sub (unique id) of the patient.

    Parameters
    ----------
    json_path : str
        The path of the JSON table file

    patient_id : str
        The value (IPP) of the new patient (ex: firstname^lastname^IPP)

    Returns:
        str : the new or already existing sub (unique id) for the patient
    """
    if not os.path.exists(json_path):
        create_anonymization_table(json_path)
        new_sub = "sub-" + str(1).zfill(FILL_NUMBER)
        write(json_path, new_sub, patient_id)
        return new_sub
    else:
        return get_sub_from_patient_id(json_path, patient_id)     

def get_sub_from_patient_id(json_path, patient_id):
    """ Look for a specific patient IPP in the anonymization table. If the patient is already in the table, it returns the unique id found in bids format of the patient.

    Parameters
    ----------
    json_path : str
        The path of the JSON table file

    patient_id : str
        The value (IPP) of the new patient (ex: firstname^lastname^IPP)

    Returns:
        str : the new or already existing sub (unique id) for the patient
    """
    with open(json_path) as f:
        data = json.load(f)
  
    liste = data["anonymisation"]
    for couple in liste:
        for sub in couple:
            if patient_id == couple[sub]:
                return sub

    for sub in liste[-1]:
        new_sub = "sub-" + str(int(sub[-3:]) + 1).zfill(FILL_NUMBER)
        write(json_path, new_sub, patient_id)
        return new_sub
    
def anonymize_dicom(dcm_path, level=1):
    """ Anonymize a Dicom file 

    Parameters
    ----------
    dcm_path : str
        The path of the Dicom file to anonymize

    level : int
        The level of anonymization (must be between 1 and 4).
        Level 1 : Patient information are removed
        Level 2 : Patient and date information are removed
        Level 3 : Patient, date and time information are removed
        Level 4 : Patient, date and time and uuid anonymized

    """
    print(f"INFO - Starting anonymization for\n\t{dcm_path :} with level {level}")
    ds = pydicom.dcmread(dcm_path, force=True)
    
    if level <= 1:
        ds.PatientName = 'Ano'
        ds.PatientID = 'Ano'
        ds.PatientBirthDate = '19000101'
        ds.PatientSex = 'Ano'
        ds.PatientAge = '00Y'
        ds.PatientSize = 0
        ds.PatientWeight = 0
    if level <= 2:
        ds.StudyDate = time.strftime("%Y%m%d")
        ds.SeriesDate = time.strftime("%Y%m%d")
        ds.AcquisitionDate = time.strftime("%Y%m%d")
        ds.ContentDate = time.strftime("%Y%m%d")
    if level <= 3:
        ds.StudyTime = time.strftime("%H%M%S")
        ds.SeriesTime = time.strftime("%H%M%S")
        ds.AcquisitionTime = time.strftime("%H%M%S")
        ds.ContentTime = time.strftime("%H%M%S")
    if level <= 4:
        uid = generate_uid()
        ds.file_meta.MediaStorageSOPInstanceUID = uid
        ds.SOPInstanceUID = uid
        
    ds.save_as(dcm_path)


import os
import pydicom
import shutil
from pydactim.anonymization import anonymize_dicom

def sort_dicom(dicom_dir, output_dir="", anonymize=False):
    """ Sort a Dicom dir by exam, patient, session and then sequence

    Parameters
    ----------
    dicom_dir : str
        Path of the directory that contains all the Dicom files to be stored

    output_dir : str
        Directory in which the Dicom files will be sorted. If blank, the 'dicom_dir' will be used as 'output_dir'

    anonymize : bool
        Whether anonymize the Dicom files or not

    """
    if output_dir == "":
        output_dir = dicom_dir

    if not os.path.exists(output_dir):
        os.makedirs(output_dir)

    dicoms_path = [file_path for file_path in os.listdir(dicom_dir) if file_path.endswith(".dcm")]
    
    for dicom_path in dicoms_path:
        dicom_path = os.path.join(dicom_dir, dicom_path)
        if anonymize: anonymize_dicom(dicom_path, level=3)

        ds = pydicom.dcmread(dicom_path)
        if "." in str(ds.PatientID):
            examen_patient_id = str(ds.PatientName)
        else:
            examen_patient_id = str(ds.PatientName).upper() + "^"+  str(ds.PatientID)

        examen_sequence_series_number = str(ds.SeriesNumber)
        examen_sequence_name = examen_sequence_series_number + " " + str(ds.SeriesDescription).upper()
        examen_study_description = str(ds.StudyDescription).upper()
        examen_date = str(ds.StudyDate)

        # on va créer le dossier protocol dans sorted_data
        protocol_dir = os.path.join(output_dir, examen_study_description)
        if not os.path.exists(protocol_dir):
            os.makedirs(protocol_dir)

        # on defini le path patient
        patient_dir = os.path.join(protocol_dir, examen_patient_id)
        if not os.path.exists(patient_dir):
            os.makedirs(patient_dir)

        # on defini le path temps d'acquisition
        time_dir = os.path.join(patient_dir, examen_date)
        if not os.path.exists(time_dir):
            os.makedirs(time_dir)

        # on defini le path sequence
        sequence_dir = os.path.join(time_dir, examen_sequence_name)
        if not os.path.exists(sequence_dir):
            os.makedirs(sequence_dir)
            print("New path created :", sequence_dir)

        shutil.copy(dicom_path, sequence_dir)
        
        



"""Mix between the MPUnet published in JMI and Unet++ declared with Pytorch"""
import numpy as np
import torch
import torch.nn as nn
import torch.nn.functional as F


def conv3d(in_channels: int, out_channels: int):
    return nn.Conv3d(in_channels=in_channels,
                     out_channels=out_channels,
                     kernel_size=3,
                     padding=1)


class MPUnetPP2CBND(nn.Module):
    def __init__(self, n_inputs: int, n_outputs: int):
        super().__init__()
        self.dropout_p = 0.3  # The default slope of leaky rely in keras is 0.3, default in pytorch is 0.01
        # Encoder
        #   First level
        self.conv000 = conv3d(n_inputs, 6)
        self.conv001 = conv3d(6, 6)
        self.drop000 = nn.Dropout3d(self.dropout_p)
        self.bn000 = nn.BatchNorm3d(6)
        self.bn001 = nn.BatchNorm3d(6)
        #   Second level
        self.conv100 = conv3d(6, 7)
        self.conv101 = conv3d(7, 7)
        self.drop100 = nn.Dropout3d(self.dropout_p)
        self.bn100 = nn.BatchNorm3d(7)
        self.bn101 = nn.BatchNorm3d(7)
        #   Third level
        self.conv200 = conv3d(7, 8)
        self.conv201 = conv3d(8, 8)
        self.drop200 = nn.Dropout3d(self.dropout_p)
        self.bn200 = nn.BatchNorm3d(8)
        self.bn201 = nn.BatchNorm3d(8)

        # Bottleneck
        self.conv300 = conv3d(8, 9)
        self.conv301 = conv3d(9, 9)
        self.drop300 = nn.Dropout3d(self.dropout_p)
        self.bn300 = nn.BatchNorm3d(9)
        self.bn301 = nn.BatchNorm3d(9)

        # Middle of Unet++
        #   First level
        #       Second column
        self.conv002 = conv3d(13, 6)
        self.conv003 = conv3d(6, 6)
        self.conv_seg0 = nn.Conv3d(in_channels=6,
                                   out_channels=n_outputs,
                                   kernel_size=1,
                                   padding=0)
        self.drop002 = nn.Dropout3d(self.dropout_p)
        self.bn002 = nn.BatchNorm3d(6)
        self.bn003 = nn.BatchNorm3d(6)
        #       Third column
        self.conv004 = conv3d(19, 6)
        self.conv005 = conv3d(6, 6)
        self.conv_seg1 = nn.Conv3d(in_channels=6,
                                   out_channels=n_outputs,
                                   kernel_size=1,
                                   padding=0)
        self.drop004 = nn.Dropout3d(self.dropout_p)
        self.bn004 = nn.BatchNorm3d(6)
        self.bn005 = nn.BatchNorm3d(6)
        #   Second level
        #       Second column
        self.conv102 = conv3d(15, 7)
        self.conv103 = conv3d(7, 7)
        self.drop102 = nn.Dropout3d(self.dropout_p)
        self.bn102 = nn.BatchNorm3d(7)
        self.bn103 = nn.BatchNorm3d(7)

        # Decoder
        #   Third level
        self.conv202 = conv3d(17, 8)
        self.conv203 = conv3d(8, 8)
        self.drop202 = nn.Dropout3d(self.dropout_p)
        self.bn202 = nn.BatchNorm3d(8)
        self.bn203 = nn.BatchNorm3d(8)
        #   Second level
        self.conv104 = conv3d(22, 7)
        self.conv105 = conv3d(7, 7)
        self.drop104 = nn.Dropout3d(self.dropout_p)
        self.bn104 = nn.BatchNorm3d(7)
        self.bn105 = nn.BatchNorm3d(7)
        #   First level
        self.conv006 = conv3d(25, 6)
        self.conv007 = conv3d(6, n_outputs)
        self.drop006 = nn.Dropout3d(self.dropout_p)
        self.bn006 = nn.BatchNorm3d(6)

        # Leaky Relu slope parameter
        self.lr_slope = 0.3  # The default slope of leaky rely in keras is 0.3, default in pytorch is 0.01

    def _activ(self, input_layer):
        # Activation function
        return F.leaky_relu(input_layer,
                            negative_slope=self.lr_slope)

    def forward(self, image):
        # Encoder
        #   First level
        fms = self.drop000(self._activ(self.bn000(self.conv000(image))))
        fms = fms_00 = self._activ(self.bn001(self.conv001(fms)))
        #   Second level
        fms = self.drop100(self._activ(self.bn100(self.conv100(F.max_pool3d(fms, (2, 2, 2))))))
        fms = fms_10 = self._activ(self.bn101(self.conv101(fms)))
        #   Third level
        fms = self.drop200(self._activ(self.bn200(self.conv200(F.max_pool3d(fms, (2, 2, 2))))))
        fms = fms_20 = self._activ(self.bn201(self.conv201(fms)))

        # Bottleneck
        fms = self.drop300(self._activ(self.bn300(self.conv300(F.max_pool3d(fms, (2, 2, 2))))))
        fms = self._activ(self.bn301(self.conv301(fms)))

        # Middle of Unet++
        #   Second level
        #       Second column
        fms_11 = self.drop102(self._activ(self.bn102(self.conv102(torch.cat((fms_10, F.interpolate(fms_20,
                                                                                                   scale_factor=2,
                                                                                                   mode='nearest')
                                                                             ), dim=1)))))

        fms_11 = self._activ(self.bn103(self.conv103(fms_11)))

        #   First level
        #       Second column
        fms_01 = self.drop002(self._activ(self.bn002(self.conv002(torch.cat((fms_00,
                                                                             F.interpolate(fms_10,
                                                                                           scale_factor=2,
                                                                                           mode='nearest')
                                                                             ), dim=1)))))
        fms_01 = self._activ(self.bn003(self.conv003(fms_01)))
        seg_01 = self.conv_seg0(fms_01)
        #       Third column
        fms_02 = self.drop004(self._activ(self.bn004(self.conv004(torch.cat((fms_00,
                                                                             fms_01,
                                                                             F.interpolate(fms_11,
                                                                                           scale_factor=2,
                                                                                           mode='nearest')
                                                                             ), dim=1)))))
        fms_02 = self._activ(self.bn005(self.conv005(fms_02)))
        seg_02 = self.conv_seg1(fms_02)

        # Decoder
        #   Third level
        fms = self.drop202(self._activ(self.bn202(self.conv202(torch.cat((fms_20,
                                                                          F.interpolate(fms,
                                                                                        scale_factor=2,
                                                                                        mode='nearest')
                                                                          ),
                                                                         dim=1)))))
        fms = self._activ(self.bn203(self.conv203(fms)))
        #   Second level
        fms = self.drop104(self._activ(self.bn104(self.conv104(torch.cat((fms_10,
                                                                          fms_11,
                                                                          F.interpolate(fms,
                                                                                        scale_factor=2,
                                                                                        mode='nearest')
                                                                          ),
                                                                         dim=1)))))
        fms = self._activ(self.bn105(self.conv105(fms)))
        #   First level
        fms = self.drop006(self._activ(self.bn006(self.conv006(torch.cat((fms_00,
                                                                          fms_01,
                                                                          fms_02,
                                                                          F.interpolate(fms,
                                                                                        scale_factor=2,
                                                                                        mode='nearest')
                                                                          ),
                                                                         dim=1)))))
        fms = torch.sigmoid(self.conv007(fms) + seg_01 + seg_02)
        return fms


if __name__ ==  '__main__':
    import nibabel as nib

    weights_path = "210510231800/MPUnetPP2CBND_epoch_29_weights.pth"
    model = MPUnetPP2CBND(1, 1)
    model.load_state_dict(torch.load(weights_path))
    model.eval()
    patch = torch.rand((1, 1, 32, 32, 32))
    affine = np.eye(4, 4)
    with torch.no_grad():
        pred = model(patch).detach().numpy()[0, 0]
    nib.save(nib.Nifti1Image(pred, affine), f'test_pred.nii.gz')



from PySide6.QtWidgets import QLabel, QFrame, QScrollArea, QMenu, QMessageBox, QCheckBox, QComboBox
from PySide6.QtCore import (
    Qt, QEvent, QSize, QPoint, QPointF, QRectF, QRect, 
)
from PySide6.QtGui import QImage, QPixmap, QPainter, QColor, QBrush, QPen, QPaintEvent, QCursor

from scipy.interpolate import interp2d
import nibabel as nib
import numpy as np

from .utils import is_dsc_param, create_lut
from .settings import *
import matplotlib.pyplot as plt

class ThumbnailFrame(QFrame):
    def __init__(self, path, replace_data, create_overlay):
        super().__init__()
        self.setFrameShape(QFrame.StyledPanel)
        self.path = path
        self.replace_data = replace_data
        self.create_overlay = create_overlay

        self.setContextMenuPolicy(Qt.CustomContextMenu)
        self.customContextMenuRequested.connect(self.showContextMenu)
        self.installEventFilter(self)  # Install event filter to handle events
        self.setCursor(Qt.PointingHandCursor)  # Set the cursor to pointing hand by default

    def showContextMenu(self, pos):
        menu = QMenu(self)

        action1 = menu.addAction("Replace image")
        action2 = menu.addAction("Add as overlay")
        
        action = menu.exec_(self.mapToGlobal(pos))
        
        if action == action1:
            self.replace_data(self.path)
        elif action == action2:
            self.create_overlay(self.path)

    def eventFilter(self, obj, event):
        if event.type() == QEvent.Type.MouseButtonRelease:
            if event.button() == Qt.LeftButton:
                self.replace_data(self.path)

        return super().eventFilter(obj, event)
    
    def enterEvent(self, event):
        # Set the cursor to pointing hand when entering the frame
        self.setCursor(Qt.PointingHandCursor)
        event.accept()

    def leaveEvent(self, event):
        # Restore the cursor to the default arrow when leaving the frame
        self.setCursor(Qt.ArrowCursor)
        event.accept()

class CustomLabel(QLabel):
    def __init__(self, path, view_data, update_slice, update_aif, update_roi, axis, size=(1000,1000)):
        super().__init__()
        self.setAlignment(Qt.AlignCenter)  # Center the image in the label
        self.setMinimumSize(size[0], size[1]) # Set the minimum size for the QLabel
        # self.setMaximumSize(size[0], size[1]) # Set the minimum size for the QLabel
        self.setStyleSheet("background-color: #000000;")
        self.setStyleSheet(f"border: 1px solid {BORDER_COLOR};")
        self.installEventFilter(self)  # Install event filter to handle events
        self.setMouseTracking(True)
        
        self.path = path
        self.data = view_data.data
        self.slice_index = view_data.slice
        self.pixdim = view_data.pixdim
        self.z = view_data.z
        self.axis = axis
        self.state = "3D"

        self.drag_start = QPoint()  # Store the starting point of the drag
        self.dragging = False  # Flag to track whether dragging is in progress
        
        self.contrast_adjustment_start = False  
        if len(np.unique(self.data)) == 2:
            print("INFO - Special windowing for binary mask has been set up")
            self.window_center = 1
            self.window_width = 1
        else:
            self.contrast_min = self.data.min()  # Minimum value for contrast windowing
            self.contrast_max = self.data.max()  # Maximum value for contrast windowing+
            self.window_center = int((self.contrast_min + self.contrast_max) / 2)  # Initial contrast center
            self.window_width = int(self.contrast_max - self.contrast_min)  # Initial contrast width

        self.zoom_factor = 1.0  # Initial zoom factor
        self.zooming = False  # Flag to track whether zooming is in progress
        
        self.panning = False
        self.pan_start = QPoint()
        self.pan_x = 0  # Initialize pan_x to 0
        self.pan_y = 0  # Initialize pan_y to 0

        self.mosaic = False

        self.mouse_current = 0
        self.mouse_threshold = 1

        self.overlay = False
        self.overlay_toggle = False
        self.overlay_path = ""
        self.overlay_data = None
        self.overlay_opacity = 50
        self.contrast_adjustment_start_overlay = False

        self.roiable = True
        self.roiing = False
        self.roi_start = QPoint()
        self.roi_x = 500
        self.roi_y = 500
        self.roi_width = ONED_HEIGHT*6/self.data.shape[0] if (is_dsc_param(self.path) or is_dsc_param(self.overlay_path)) else 30

        self.update_slice = update_slice
        self.update_aif = update_aif
        self.update_roi = update_roi
        self.selected_widget = "info_widget"

    def set_overlay(self, path, data, pixdim, lut):
        self.overlay = True
        self.overlay_toggle = True
        self.overlay_path = path
        self.overlay_data = data
        self.overlay_pixdim = pixdim
        self.overlay_lut = create_lut(lut)

        if len(np.unique(self.overlay_data)) == 2:
            print("INFO - Special windowing for binary mask has been set up")
            self.window_center_overlay = 1
            self.window_width_overlay = 1
        else:
            self.contrast_min_overlay = self.data.min()  # Minimum value for contrast windowing
            self.contrast_max_overlay = self.data.max()  # Maximum value for contrast windowing+
            self.window_center_overlay = int((self.contrast_min_overlay + self.contrast_max_overlay) / 2)  # Initial contrast center
            self.window_width_overlay = int(self.contrast_max_overlay - self.contrast_min_overlay)  # Initial contrast width
        self.update_image()

    def update_image(self):
        current_data = self.data
        # current_data =  
        current_data = (current_data - (self.window_center - self.window_width / 2)) / (self.window_width + 1e-12) * 255

        current_data = current_data.astype('uint8')
        current_data = self.lut[current_data]
        current_data = np.ascontiguousarray(current_data)

        # Create a QImage from the slice data
        self.qimage = QImage(current_data, current_data.shape[1], current_data.shape[0], current_data.shape[1]*3, QImage.Format_RGB888)
        pixmap = QPixmap.fromImage(self.qimage)

        pixel_width, pixel_height, _ = self.pixdim
        scaled_width = pixmap.width() * pixel_width
        scaled_height = pixmap.height() * pixel_height
        pixmap = pixmap.scaled(scaled_width, scaled_height, Qt.IgnoreAspectRatio, Qt.SmoothTransformation)

        # # Resize the pixmap to fit the application window with some scaling factor (e.g., 2 times larger)
        self.scale_factor = 1 / self.zoom_factor
        pixmap = pixmap.scaled(self.size() * self.scale_factor, Qt.KeepAspectRatio, Qt.SmoothTransformation)

        if self.state == "3D":
            if pixmap.width() > THREED_WIDTH or pixmap.height() > THREED_HEIGHT:
                if pixmap.width() > pixmap.height():
                    new_width = THREED_WIDTH
                    new_height = int(pixmap.height() * (THREED_WIDTH / pixmap.width()))
                else:
                    new_height = THREED_HEIGHT
                    new_width = int(pixmap.width() * (THREED_HEIGHT / pixmap.height()))

                pixmap = pixmap.scaled(new_width, new_height, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        else:
            pixmap = pixmap.scaled(ONED_WIDTH, ONED_HEIGHT, Qt.KeepAspectRatio, Qt.SmoothTransformation)

        self.pixmap_width = pixmap.width()
        self.pixmap_height = pixmap.height()

        if self.overlay and self.overlay_toggle:
            current_overlay = self.overlay_data
            current_overlay = np.clip(current_overlay, self.window_center_overlay - self.window_width_overlay / 2, self.window_center_overlay + self.window_width_overlay / 2)
            current_overlay = (current_overlay - (self.window_center_overlay - self.window_width_overlay / 2)) / (self.window_width_overlay + 1e-12) * 255

            current_overlay = current_overlay.astype('uint8')
            current_overlay = self.overlay_lut[current_overlay]
            current_overlay = np.ascontiguousarray(current_overlay)

            current_overlay = QImage(current_overlay.data, current_overlay.shape[1], current_overlay.shape[0], current_overlay.shape[1]*3, QImage.Format_RGB888)
            pixel_width, pixel_height, _ = self.overlay_pixdim
            scaled_width = int(current_overlay.width() * pixel_width)
            scaled_height = int(current_overlay.height() * pixel_height)
            current_overlay = current_overlay.scaled(scaled_width, scaled_height, Qt.IgnoreAspectRatio, Qt.SmoothTransformation)
            current_overlay = current_overlay.scaled(pixmap.size(), Qt.KeepAspectRatio, Qt.SmoothTransformation)
            
            painter = QPainter()
            painter.begin(pixmap)
            painter.setOpacity(self.overlay_opacity / 100)
            painter.drawImage(0, 0, current_overlay)
            painter.setBackgroundMode(Qt.OpaqueMode)
            painter.end()

        if ("perf." in self.path or "t2star." in self.path) and self.roiable and self.selected_widget == "perf_widget":
            self.painterInstance = QPainter(pixmap)
            self.penRectangle = QPen(Qt.red)
            self.penRectangle.setWidth(2)

            # draw rectangle on painter
            self.painterInstance.setPen(self.penRectangle)
            self.painterInstance.drawRect(self.roi_x, self.roi_y, self.roi_width, self.roi_width)
            self.painterInstance.end()

        elif (is_dsc_param(self.path) or is_dsc_param(self.overlay_path)) and self.roiable and self.selected_widget == "perf_widget":
            self.painterInstance = QPainter(pixmap)
            self.penRectangle = QPen(Qt.white)
            self.penRectangle.setWidth(2)

            # draw rectangle on painter
            self.painterInstance.setPen(self.penRectangle)
            self.painterInstance.drawRect(self.roi_x, self.roi_y, self.roi_width, self.roi_width)
            self.painterInstance.end()
            
        self.pixmap = pixmap
        self.setPixmap(self.pixmap)
    
    def eventFilter(self, obj, event):
        if event.type() == QEvent.Type.MouseButtonPress:
            if event.button() == Qt.RightButton:
                # Start image dragging when right button is pressed
                self.drag_start = event.position()
                self.dragging = True
            elif event.button() == Qt.LeftButton:
                # Start image dragging when left button is pressed
                self.drag_start = event.position()
                label_width = self.width()
                x_pos = event.position().x()
                y_pos = event.position().y()
                # if 0 < x_pos < 0.1 * label_width or 0.9 * label_width < x_pos < label_width:
                #     self.zooming = True
                if x_pos > self.roi_x and x_pos < self.roi_x + self.roi_width:
                    if self.pixmap_height == ONED_HEIGHT:
                        offset = 41
                    else:
                        offset = 16
                    if y_pos > self.roi_y + offset and y_pos < self.roi_y + offset + self.roi_width:
                        self.roiing = True
                        self.roi_start = event.position()
                        self.roi_start_x = self.roi_x
                        self.roi_start_y = self.roi_y
                # else:
                #     self.panning = True
                #     self.pan_start = event.position()
                #     self.pan_start_x = self.pan_x
                #     self.pan_start_y = self.pan_y
            elif event.button() == Qt.MiddleButton:
                # Start contrast adjustment when middle button is pressed
                self.drag_start = event.position()
                self.contrast_adjustment_start = self.window_center, self.window_width
                if self.overlay:
                    self.contrast_adjustment_start_overlay = self.window_center_overlay, self.window_width_overlay

        # MOUSE RELEASE EVENT
        elif event.type() == QEvent.Type.MouseButtonRelease:
            self.mouse_current = 0
            if event.button() == Qt.RightButton:
                # End image dragging when right button is released
                self.dragging = False
            elif event.button() == Qt.LeftButton:
                if self.zooming:
                    self.zooming = False
                elif self.panning:
                    self.panning = False
                elif self.roiing:
                    self.roiing = False
            elif event.button() == Qt.MiddleButton:
                # End contrast adjustment when left button is released
                self.contrast_adjustment_start = None

        # MOUSE WHEEL SCROLL EVENT
        elif event.type() == QEvent.Type.Wheel:
            # Handle wheel event for scrolling
            num_degrees = event.angleDelta().y() / 8
            num_steps = num_degrees / 15
            new_index = self.slice_index + num_steps
            new_index = max(0, min(new_index, self.z - 1))
            if new_index != self.slice_index:
                self.update_slice(int(new_index), self.axis)
                self.update_image()
                self.update_aif()
                self.compute_roi()

        # MOUSE RIGHT CLICK FOR SLICES
        elif event.type() == QEvent.Type.MouseMove and self.dragging:
            # Handle image dragging when right button is pressed and mouse is moved
            delta = self.drag_start - event.position()
            num_steps = delta.y()
            new_index = self.slice_index + num_steps 
            new_index = max(0, min(new_index, self.z - 1))
            if new_index != self.slice_index:
                self.update_slice(int(new_index), self.axis)
                self.update_image()
                self.update_aif()
                self.compute_roi()
            self.drag_start = event.position()

            # delta = self.drag_start - event.position()
            # self.mouse_current += delta.y()
            # print(f"{self.mouse_current =}")
            # if self.mouse_current >= self.mouse_threshold:
            #     print(f"++++ {self.slice_index =}; {self.z =}")
            #     self.slice_index = min(self.slice_index + 1, self.z - 1)
            #     self.update_slice(self.slice_index, self.axis)
            #     print(f"New {self.slice_index =}")
            #     self.update_image()
            #     self.update_aif()
            #     self.mouse_current = 0
            # elif self.mouse_current <= -self.mouse_threshold:
            #     print(f"----{self.slice_index =}; {self.z =}")
            #     self.slice_index = max(0, self.slice_index - 1)
            #     print(f"New {self.slice_index =}")
            #     self.update_slice(self.slice_index, self.axis)
            #     self.update_image()
            #     self.update_aif()
            #     self.mouse_current = 0
            # self.drag_start = event.position()

        # MOUSE WHEEL CLICK FOR WINDOWING
        elif event.type() == QEvent.Type.MouseMove and self.contrast_adjustment_start:
            # Handle contrast adjustment when left button is pressed and mouse is moved
            delta = self.drag_start - event.position()
            num_steps_x = int(delta.x() * 1.1)
            num_steps_y = int(delta.y() * 1.1)

            # Calculate new window center and width based on the mouse movement
            if not self.overlay_toggle:
                self.window_width = int(self.window_width - num_steps_x) 
                if self.window_width < 1: self.window_width = 1
                self.window_center = int(self.window_center + num_steps_y)
                if self.window_center < 0: self.window_center = 0
            else:
                self.window_width_overlay = int(self.window_width_overlay - num_steps_x) 
                if self.window_width_overlay < 1: self.window_width_overlay = 1
                self.window_center_overlay = int(self.window_center_overlay + num_steps_y)
                if self.window_center_overlay < 0: self.window_center_overlay = 0

            self.drag_start = event.position()
            self.update_image()
            self.update_aif()

        # # MOUSE LEFT CLICK FOR ROI MOVEMENT
        elif event.type() == QEvent.Type.MouseMove and self.roiing and self.roiable:
            delta = event.position() - self.roi_start
            self.roi_x = self.roi_start_x - (-1*delta.x())
            self.roi_y = self.roi_start_y - (-1*delta.y())
            self.update_image()
            self.update_aif()
            self.compute_roi()

        # elif event.type() == QEvent.Type.MouseMove and self.zooming:
        #     # Handle zooming when left button is pressed and mouse is moved
        #     label_width = self.width()

        #     delta = self.drag_start - event.position()
        #     self.zoom_factor += delta.y() * -0.005

        #     # Calculate the new zoom factor based on the mouse movement
        #     if self.zoom_factor > 1.0: self.zoom_factor = 1.0
        #     if self.zoom_factor < 0.3: self.zoom_factor = 0.3

        #     self.drag_start = event.position()
        #     self.update_image()

        # # MOUSE LEFT CLICK FOR PANNING
        # elif event.type() == QEvent.Type.MouseMove and self.panning:
        #     # Handle panning when left button is pressed and mouse is moved
        #     delta = event.position() - self.pan_start
        #     self.pan_x = self.pan_start_x - (-1*delta.x())
        #     self.pan_y = self.pan_start_y - (-1*delta.y())
        #     self.update_image()
        
        # if event.type() == QEvent.Type.MouseMove and self.roiing and self.roiable and (is_dsc_param(self.path) or is_dsc_param(self.overlay_path)):
          

            
        
        # elif event.type() == QEvent.Leave:
        #     print("Mouse is not over the label", event.pos())

        return super().eventFilter(obj, event)

    def enterEvent(self, event):
        self.setStyleSheet(f"border: 1px solid {TEXT_COLOR};")

    # def mouseMoveEvent(self, event):
    #     # Get the mouse position relative to the top-left corner of the label
    #     global_mouse_pos = QCursor.pos()

    #     # Convert global mouse position to local coordinates of the label
    #     local_mouse_pos = self.mapFromGlobal(global_mouse_pos)
    #     x = local_mouse_pos.x()
    #     y = local_mouse_pos.y()
        # Do something with the mouse position
        # print("################################")
        # print(f"{self.axis}: ({local_mouse_pos.x()}, {local_mouse_pos.y()})")
        # print(QColor(self.qimage.pixel(x,y)).getRgb())
        

    def leaveEvent(self, event):
        self.setStyleSheet(f"border: 1px solid {LIGHT_BORDER_COLOR};")

    def compute_roi(self):
        if self.selected_widget == "perf_widget":
            if is_dsc_param(self.overlay_path):
                data = self.overlay_data
                pixdim = self.overlay_pixdim
            else: 
                data = self.data
                pixdim = self.pixdim

            x = self.roi_x + self.roi_width / 2
            y = self.roi_y + 41 + self.roi_width / 2
            self.roi_xx = x * data.shape[1] // self.pixmap_width
            if self.pixmap_height == ONED_HEIGHT:
                self.roi_yy = (self.pixmap_height - y + 41) * data.shape[0] // self.pixmap_height
            else: 
                self.roi_yy = (self.pixmap_height - y + 16) * data.shape[0] // self.pixmap_height
            
            self.roi = data[round(self.data.shape[0]-self.roi_yy - 6//2):round(self.data.shape[0]-self.roi_yy + 6//2), round(self.roi_xx - 6//2):round(self.roi_xx + 6//2)]
            
            # DEBUG
            # fig, ax = plt.subplots()
            # vmin = np.min(data)
            # vmax = np.max(data)
            # ax.imshow(self.roi, vmin=vmin , vmax=vmax , cmap='turbo')
            # plt.savefig("temp.png")
            # plt.close()
            
            self.roi = self.roi.flatten()
            self.roi_mean = np.mean(self.roi)
            self.roi_std = np.std(self.roi)
            self.roi_min = np.min(self.roi)
            self.roi_max = np.max(self.roi)
            self.roi_mm3 = len(self.roi) * pixdim[0] * pixdim[1]

            self.update_roi()

class ScrollArea(QScrollArea):
    def __init__(self):
        super().__init__()
        self.setMinimumSize(SCROLL_WIDTH, WINDOW_HEIGHT)
        self.setMaximumSize(SCROLL_WIDTH, WINDOW_HEIGHT)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)  # Hide horizontal scrollbar
        self.setWidgetResizable(True)

    def keyPressEvent(self, event):
        if event.key() == Qt.Key_Right:
            event.ignore()
        elif event.key() == Qt.Key_Left:
            event.ignore()
        else:
            return super().keyPressEvent(event)

class ComboBox(QComboBox):
    def __init__(self):
        super().__init__()

    def keyPressEvent(self, event):
        if event.key():
            event.ignore()
        else:
            super().keyPressEvent(event)

class AnimatedToggle(QCheckBox):
    def __init__(self, toggle_overlay, width=40):
        QCheckBox.__init__(self)

        self.setFixedSize(width, 22)
        self.setCursor(Qt.PointingHandCursor)

        self._bg_color = LIGHT_BORDER_COLOR
        self._circle_color = TEXT_COLOR
        self._active_color = VERY_LIGHT_BG_COLOR

        self.toggle_overlay = toggle_overlay
        self.setChecked(True)

        self.stateChanged.connect(self.toggle_overlay)

    def debug(self):
        print(f"Status: {self.isChecked()}")

    def hitButton(self, pos: QPoint):
        return self.contentsRect().contains(pos)

    def paintEvent(self, event):
        p = QPainter(self)
        p.setRenderHint(QPainter.Antialiasing)
        p.setPen(Qt.NoPen)

        height = 20

        rect = QRect(0,0,self.width(), height)

        if not self.isChecked():
            p.setBrush(QColor(self._bg_color))
            p.drawRoundedRect(0,0,rect.width(), height, height / 2, height / 2)

            p.setBrush(QColor(self._circle_color))
            p.drawEllipse(3,2,18,18)
        else:
            p.setBrush(QColor(self._active_color))
            p.drawRoundedRect(0,0,rect.width(), height, height / 2, height / 2)

            p.setBrush(QColor(self._circle_color))
            p.drawEllipse(self.width() - 20,1,18,18)

        p.end()


from .transformation import (
    n4_bias_field_correction,
    registration,
    apply_transformation,
    resample,
    histogram_matching,
    apply_mask,
    substract,
    susan,
    normalize,
    crop,
    apply_crop,
    copy_affine,
    skull_stripping,
    remove_small_object,
    tissue_classifier,
    add_tissue_class,
    prediction_glioma,
    uncertainty_prediction_glioma,
    extract_dim
)
from .sorting import sort_dicom
from .conversion import convert_dicom_to_nifti
from .visualization import plot, plot_histo
from .data import MRIData
from .computation import calc_volume
from .utils import load_dicom, timed, is_native
from .preprocessing import preproc, init
from .viewer.main import ViewerApp


import os, time
from functools import wraps

def timed(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        start = time.time()
        result = func(*args, **kwargs)
        end = time.time()

        seconds = round(end - start, 2)
        minutes = int((seconds % 3600) // 60)
        hours = int(seconds // 3600)
        remaining_seconds = round(seconds % 60)

        # Build the formatted time string
        formatted_time = ""
        if hours > 0:
            formatted_time += f"{hours}h"
        if minutes > 0:
            formatted_time += f"{minutes}min"
        if remaining_seconds > 0:
            formatted_time += f"{remaining_seconds}sec"
        
            
        print(f"INFO - {func.__name__} ran in {formatted_time}")
        return result
    return wrapper

def is_native(path):
    path = os.path.basename(path)
    temp = path.split("_")[2:]
    if   temp[0] == "ce-GADOLINIUM": temp = temp[1:]
    elif temp[0] == "task"    and temp[1] == "rest":   temp = temp[1:]
    elif temp[0] == "dwi"     and temp[1] == "pa":     temp = temp[1:]
    elif temp[0] == "resolve" and temp[1] == "tracew": temp = temp[1:]
    elif temp[0] == "resolve" and temp[1] == "adc":    temp = temp[1:]
    if len(temp) > 1: return False
    return True
    
def is_useful(path):
    path = os.path.basename(path)
    temp = path.split("_")[2:]
    if   temp[0] == "ce-GADOLINIUM": temp = temp[1:]
    elif temp[0] == "task"    and temp[1] == "rest":   temp = temp[1:]
    elif temp[0] == "dwi"     and temp[1] == "pa":     temp = temp[1:]
    elif temp[0] == "resolve" and temp[1] == "tracew": temp = temp[1:]
    elif temp[0] == "resolve" and temp[1] == "adc":    temp = temp[1:]
    seq = temp[0].split(".")[0]
    not_useful = ["rest", "B0map", "perf", "dwi", "pa", "tracew"]
    if seq in not_useful:
        return False
    return True

def get_name_of_path(path):
    if not "sub" in path and not "ses" in path:
        return path
    else:
        return " ".join(path.replace(".nii.gz", "").replace(".nii", "").split("_")[2:])

def load_dicom(dicom_path):
    """Read a dicom as a binary (CSA Header) and create a dictionnary from each sequence parameter

    Args:
        dicom_path (str) : the absolute path of the dicom

    Returns:
        dict : the dictionnary that holds all the sequence parameters
    """
    f = str(open(dicom_path, 'rb').readlines())
    f = f[f.find("b'### ASCCONV BEGIN"):f.find(", b'### ASCCONV END ###")]
    f = f.replace("\\n", "").replace("\\t", "").replace(" b'", "").replace("'", "").replace("NE1,", "NE1 ").split(",")
    dcm = {row.split(' = ')[0] : row.split(' = ')[1] for row in f[1:]}
    return dcm

if __name__ == '__main__':
    print(get_name_of_path(r"D:\Results\GLIOBIOPSY\derivative\sub-003\ses-01\anat\sub-003_ses-01_FLAIR_brain_1mm.nii.gz"))


import nibabel as nib
import numpy as np

def calc_volume(mask_path):
    """ Calculate the volume of a mask 

    Parameters
    ----------
    mask_path : str
        Path of the mask

    """
    mask = nib.load(mask_path)
    pixdim = mask.header["pixdim"][1:4]
    volume = np.count_nonzero(mask.get_fdata())
    volumetrie = volume * pixdim[0] * pixdim[1] * pixdim[2]
    print("Volume :", round(volumetrie, 3), "cm3")
    return round(volumetrie/1000,3)

def a_calc_volume(mask_data, pixdim):
    """ Calculate the volume of a mask

    Parameters
    ----------
    mask_data : array
        Array of the mask

    pixdim : array
            Array of size 3 of the dimension of the voxels

    """
    volume = np.count_nonzero(mask_data)
    volumetrie = volume * pixdim[0] * pixdim[1] * pixdim[2]
    print("Volume :", round(volumetrie, 3), "cm3")
    return round(volumetrie/1000,3)